import 'package:stub/stub.dart';
import 'package:test/test.dart';

void main() {
  group('`Stub` NULLARY test', () {
    final function = nullaryStub<int>();
    const data = 1;
    const times = 42;
    test(
        'GIVEN `stub is not set` '
        'WHEN function.stub is called '
        'THEN throw `UnimplementedError`', () {
                function.reset();

      expect(
        () => function(),
        throwsA(isA<UnimplementedError>()),
        reason: '`function.stub` should throw `UnimplementedError`',
      );
    });

    test(
        'GIVEN `stub = () => data` '
        'WHEN function.stub is called '
        'THEN returns `data`', () {
      function.reset();
      function.stub = () => data;

      expect(
        function(),
        data,
        reason: '`function.stub` should match `data`',
      );
    });
    test(
        'GIVEN `stub != null` '
        'WHEN function.stub is called n times '
        'THEN  `count` == `times`', () {
      function.reset();

      for (var i = 0; i < times; ++i) {
        function();
      }
      expect(
        function.count,
        times,
        reason: '`function.stub` should match `times`',
      );
    });
    test(
        '...'
        'WHEN  function.stub is not called '
        'THEN  `callCount` persists', () {

      expect(
        function.count,
        times,
        reason: '`function.stub` should persist across tests',
      );
    });
    test(
        'GIVEN _ '
        'WHEN Foo.foo is set twice '
        'THEN  only latest has effect', () {
      function.reset();
      function.stub = () => data * 3;
      function.stub = () => data * times;

      expect(
        function(),
        data * times,
        reason: '`function.stub()` should be the latest `result` provided',
      );
    });
  });
}
